/** \file Main.h
*/

#ifndef MainH
#define MainH
//---------------------------------------------------------------------------
#include "display/Display.h"
#include "Mutex.h"

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <ComCtrls.hpp>
#include <Menus.hpp>
#include <ToolWin.hpp>
#include <ImgList.hpp>
#include <ActnList.hpp>

class DeviceInterface;

//---------------------------------------------------------------------------
class TfrmMain : public TForm
{
__published:	// IDE-managed Components
	TPanel *pnlRight;
	TPanel *pnlCenter;
	TTimer *Timer;
	TButton *btnStartStop;
	TTrackBar *TrackBarSpeed;
	TStatusBar *StatusBar;
	TMainMenu *Menu;
	TMenuItem *File1;
	TMenuItem *miHelp;
	TMenuItem *E1;
	TMenuItem *miSettings;
	TImageList *imglToolbar;
	TCoolBar *CoolBar;
	TToolBar *ToolBarMode;
	TToolButton *btnModeScope;
	TToolButton *btnModeSpectrum;
	TToolButton *btnModeRecorder;
	TToolBar *ToolBarMouse;
	TToolButton *btnMouseCursor;
	TToolButton *btnMouseZoomXY;
	TToolButton *btnMouseMove;
	TActionList *ActionList;
	TAction *ActionChangeMouseModeCursor;
	TAction *ActionChangeMouseModeOffset;
	TAction *ActionChangeMouseModeZoom;
	TToolButton *btnMouseZoomX;
	TToolButton *btnMouseZoomY;
	TAction *ActionChangeMouseModeZoomX;
	TAction *ActionChangeMouseModeZoomY;
	TToolButton *btnZoomClear;
	TAction *ActionZoomClear;
	TAction *ActionModeScope;
	TAction *ActionModeSpectrum;
	TMenuItem *miAbout;
	TMenuItem *miCommonSettings;
	TMenuItem *miDeviceSettings;
	TMenuItem *miView;
	TMenuItem *miViewLog;
	TToolBar *ToolBarAcquire;
	TToolButton *btnAcquireStop;
	TToolButton *btnAcquireStart;
	TAction *ActionAcquireStart;
	TAction *ActionAcquireStop;
	TLabel *lblRefreshing;
	void __fastcall FormDestroy(TObject *Sender);
	void __fastcall pnlCenterResize(TObject *Sender);
	void __fastcall TimerTimer(TObject *Sender);
	void __fastcall btnStartStopClick(TObject *Sender);
	void __fastcall TrackBarSpeedChange(TObject *Sender);
	void __fastcall pnlCenterMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall ActionChangeMouseModeCursorExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeOffsetExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomXExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomYExecute(TObject *Sender);
	void __fastcall ActionZoomClearExecute(TObject *Sender);
	void __fastcall pnlCenterMouseUp(TObject *Sender, TMouseButton Button,
		  TShiftState Shift, int X, int Y);
	void __fastcall ActionModeScopeExecute(TObject *Sender);
	void __fastcall ActionModeSpectrumExecute(TObject *Sender);
	void __fastcall miAboutClick(TObject *Sender);
	void __fastcall miCommonSettingsClick(TObject *Sender);
	void __fastcall FormCreate(TObject *Sender);
	void __fastcall miDeviceSettingsClick(TObject *Sender);
	void __fastcall miViewLogClick(TObject *Sender);
	void __fastcall ActionAcquireStartExecute(TObject *Sender);
	void __fastcall ActionAcquireStopExecute(TObject *Sender);
private:	// User declarations
	Display *display;
	enum E_MOUSE_TOOL { MOUSE_CURSOR = 0,
						MOUSE_OFFSET,
						MOUSE_ZOOM,
						MOUSE_ZOOM_X,
						MOUSE_ZOOM_Y } mouse_tool;
	enum E_CURSOR_RES { CURSOR_HAND = 100,
						CURSOR_ZOOM };
	struct DISP_STATE {
		float fZoomX;
		float fZoomY;
	} displayOsc, displaySpe;
	inline struct DISP_STATE& DisplayState(void) {
		if (display->GetMode() == MODE_OSCILLOSCOPE) {
			return displayOsc;
		} else {
			return displaySpe;
		}
	}
	const float fZoomChangeGrid;
	void UpdateCursorReadings(enum E_DISPLAY_MODE, int cursorId, float fValX, float fValY);
	void CalculateSpectrum(void);
	int iPlotMoveStartX, iPlotMoveStartY;	///< temporary variables used to pass offset vector to display
	bool bMovingPlot;
	bool bDataUpdate;						///< flag set when new data is acquired
	std::vector<float> data, dataFFT;
	DeviceInterface *device;
	Mutex mutexDeviceData;
public:		// User declarations
	__fastcall TfrmMain(TComponent* Owner);
	__fastcall ~TfrmMain();
	void CallbackCloseLogFn(void);
	void CallbackDataFn(short*, int);
};
//---------------------------------------------------------------------------
extern PACKAGE TfrmMain *frmMain;
//---------------------------------------------------------------------------
#endif
