/** \file MiniscopeDevice.h
	\brief Interface description for miniscope project device (ADC, preamplifier...) plugins
	\author Tomasz Ostrowski, http://tomeko.net
	\date October 2008
*/

#ifndef MiniscopeDeviceH
#define MiniscopeDeviceH
//---------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>
#include "MiniscopeDeviceCapabilities.h"
#include "MiniscopeDeviceCommon.h"

// export/import declarations switch
// define _EXPORTING when including file in dll

#ifdef _EXPORTING
#define DECLARE_FUNCTION(fn) __declspec(dllexport) void
#define DECLARE_FUNCTION(type, fn, __VA_ARGS__) __declspec(dllexport) type __stdcall fn(__VA_ARGS__)
#else
#define DECLARE_FUNCTION(type, fn, __VA_ARGS__) __declspec(dllimport) type __stdcall fn(__VA_ARGS__); \
	typedef type __stdcall (*pf##fn)(__VA_ARGS__)
#endif

#ifdef _EXPORTING
	#define DECLSPEC __declspec(dllexport)
#else
    #define DECLSPEC __declspec(dllimport)
#endif

enum { DLL_INTERFACE_MAJOR_VERSION = 0 };
enum { DLL_INTERFACE_MINOR_VERSION = 1 };

enum E_CONNECTION_STATE
{
    DEVICE_CONNECTED = 0,
    DEVICE_DISCONNECTED
};

struct S_SCOPE
{
    int iId;
    char szDescription;
};

// callback declarations
typedef void __stdcall (*CALLBACK_LOG)(char *szText);                   ///< called to generate log in parent application
typedef void __stdcall (*CALLBACK_CONNECT)(int state, char *szMsgText); ///< called on connection/disconnection of device
typedef void __stdcall (*CALLBACK_DATA)(void *buffer, unsigned int buffersize); ///< called to pass data (samples) to parent application
typedef void __stdcall (*CALLBACK_TRIGGER)(void);

// exported/imported function set
DECLSPEC void __stdcall SetCallbacks(
    CALLBACK_LOG lpLog,
    CALLBACK_CONNECT lpConnect,
    CALLBACK_DATA lpData,
    CALLBACK_TRIGGER lpTrigger
);
typedef void __stdcall (*pfSetCallbacks)(
    CALLBACK_LOG lpLog,
    CALLBACK_CONNECT lpConnect,
    CALLBACK_DATA lpData,
    CALLBACK_TRIGGER lpTrigger
);

DECLARE_FUNCTION(void, GetMiniscopeInterfaceDescription, struct S_DEVICE_DLL_INTERFACE*);

//DECLSPEC void __stdcall GetMiniscopeInterfaceDescription(struct S_DEVICE_DLL_INTERFACE*);
//typedef void __stdcall (*pfGetMiniscopeInterfaceDescription)(struct S_DEVICE_DLL_INTERFACE*);

/** \brief Get list of all available supported devices */
DECLSPEC int __stdcall EnumerateDevices(struct S_SCOPE *devs);
DECLSPEC int __stdcall ConnectDevice(int iId);
DECLSPEC int __stdcall DisconnectDevice();
DECLSPEC void __stdcall SetLogLevel(int level);
DECLSPEC void __stdcall GetDeviceCapabilities(struct S_SCOPE_CAPABILITIES *caps);

DECLSPEC int __stdcall ActivateChannel(int iId);
DECLSPEC int __stdcall SetSensitivity(int iChannel, int iId);
DECLSPEC int __stdcall SetSamplingPeriod(int iId);
DECLSPEC int __stdcall SetTriggerType(int iChannel, int iId);
DECLSPEC int __stdcall SetCouplingType(int iChannel, int iId);
DECLSPEC int __stdcall SetBufferSize(int iId);
DECLSPEC int __stdcall SetOffset(float fOffset);
DECLSPEC int __stdcall SetTriggerSource(int i);

DECLSPEC int __stdcall AcquireStart(void);
DECLSPEC int __stdcall AcquireStop(void);
DECLSPEC int __stdcall ManualTrigger(void);



#ifdef __cplusplus
}
#endif

#endif
